@@ -2,6 +2,7 @@
 
 Useful base class for gustaf. Plus some useful decorators.
 """
+
 from gustaf.utils import log
 
 
@@ -10,7 +11,7 @@ class GustafBase:
     methods are defined as classmethods..
     """
 
-    __slots__ = ()
+    __slots__ = ("__weakref__",)
 
     def __init_subclass__(cls, *args, **kwargs):
         """
