@@ -8,7 +8,7 @@ from gustaf import create, utils
 from gustaf.volumes import Volumes
 
 
-def box(bounds=[[0.0, 0.0, 0.0], [1.0, 1.0, 1.0]], resolutions=[2, 2, 2]):
+def box(bounds=None, resolutions=None):
     """Create structured hexahedron block mesh.
 
     Parameters
@@ -23,6 +23,10 @@ def box(bounds=[[0.0, 0.0, 0.0], [1.0, 1.0, 1.0]], resolutions=[2, 2, 2]):
     volume_mesh: Volumes
     """
 
+    if resolutions is None:
+        resolutions = [2, 2, 2]
+    if bounds is None:
+        bounds = [[0.0, 0.0, 0.0], [1.0, 1.0, 1.0]]
     if np.array(bounds).shape != (2, 3):
         raise ValueError("Bounds must have a dimension of (2, 3).")
     if len(resolutions) != 3:
