@@ -2,6 +2,7 @@
 
 Edges. Also known as lines.
 """
+
 from copy import deepcopy
 
 import numpy as np
@@ -18,7 +19,12 @@ class EdgesShowOption(helpers.options.ShowOption):
 
     _valid_options = helpers.options.make_valid_options(
         *helpers.options.vedo_common_options,
-        Option("vedo", "lw", "Width of edges (lines) in pixel units.", (int,)),
+        Option(
+            "vedo",
+            "lw",
+            "Width of edges (lines) in pixel units.",
+            (float, int),
+        ),
         Option("vedo", "as_arrows", "Show edges as arrows.", (bool,)),
         Option(
             "vedo",
@@ -42,7 +48,7 @@ class EdgesShowOption(helpers.options.ShowOption):
 
     _helps = "Edges"
 
-    def _initialize_vedo_showable(self):
+    def _initialize_showable(self):
         """
         Initializes edges as either vedo.Lines or vedo.Arrows
 
@@ -85,7 +91,6 @@ class Edges(Vertices):
         vertices=None,
         edges=None,
         elements=None,
-        copy=True,
     ):
         """Edges. It has vertices and edges. Also known as lines.
 
@@ -94,7 +99,7 @@ class Edges(Vertices):
         vertices: (n, d) np.ndarray
         edges: (n, 2) np.ndarray
         """
-        super().__init__(vertices=vertices, copy=copy)
+        super().__init__(vertices=vertices)
 
         if edges is not None:
             self.edges = edges
@@ -132,7 +137,7 @@ class Edges(Vertices):
         self._logd("setting edges")
 
         self._edges = helpers.data.make_tracked_array(
-            es, settings.INT_DTYPE, self.setter_copies
+            es, settings.INT_DTYPE, copy=False
         )
 
         # shape check
@@ -250,14 +255,14 @@ class Edges(Vertices):
         return getattr(self, elem_name)
 
     @elements.setter
-    def elements(self, elems):
+    def elements(self, elements):
         """Calls corresponding connectivity setter. A short cut in FEM friendly
         term. Vertices -> vertices Edges -> edges Faces -> faces Volumes ->
         volumes.
 
         Parameters
         -----------
-        elems: (n, d) np.ndarray
+        elements: (n, d) np.ndarray
 
         Returns
         --------
@@ -265,8 +270,8 @@ class Edges(Vertices):
         """
         # naming rule in gustaf
         elem_name = type(self).__qualname__.lower()
-        self._logd(f"seting {elem_name}'s connectivity.")
-        return setattr(self, elem_name, elems)
+        self._logd(f"Setting {elem_name}'s connectivity.")
+        return setattr(self, elem_name, elements)
 
     @property
     def const_elements(self):
@@ -365,7 +370,7 @@ class Edges(Vertices):
 
         ``dashed edges``
 
-        .. code-block::
+        .. code-block:: text
 
              o--------o    o--------o    o--------o
              |<------>|             |<-->|
@@ -398,7 +403,7 @@ class Edges(Vertices):
         for v0, v1, lins in zip(v0s, v1s, linspaces):
             new_vs.append(np.linspace(v0, v1, lins))
 
-        # we need all choped vertices.
+        # we need all chopped vertices.
         # there might be duplicating vertices. you can use merge_vertices
         new_vs = np.vstack(new_vs)
         # all mid points are explicitly defined, but they aren't required
@@ -413,15 +418,15 @@ class Edges(Vertices):
 
         return Edges(vertices=new_vs, edges=new_es)
 
-    def shrink(self, ratio=0.8, map_vertexdata=True):
+    def shrink(self, ratio=0.8, map_vertex_data=True):
         """Returns shrunk elements.
 
         Parameters
         -----------
         ratio: float
           Default is 0.8
-        map_vertexdata: bool
-          Default is True. Maps all vertexdata.
+        map_vertex_data: bool
+          Default is True. Maps all vertex_data.
 
         Returns
         --------
@@ -443,19 +448,19 @@ class Edges(Vertices):
 
         s_elements = type(self)(vertices=vs, elements=es)
 
-        if map_vertexdata:
+        if map_vertex_data:
             elements_flat = elements.ravel()
-            for key, value in self.vertexdata.items():
-                s_elements.vertexdata[key] = value[elements_flat]
+            for key, value in self.vertex_data.items():
+                s_elements.vertex_data[key] = value[elements_flat]
 
-            # probably wanna take visulation options too
+            # probably wanna take visualization options too
             s_elements._show_options._options = deepcopy(
                 self.show_options._options
             )
 
         return s_elements
 
-    def tovertices(self):
+    def to_vertices(self):
         """Returns Vertices obj.
 
         Parameters
