@@ -3,62 +3,75 @@
 Helps helpee to manage data. Some useful data structures.
 """
 
-
 from collections import namedtuple
 from functools import wraps
 
 import numpy as np
 
-from gustaf._base import GustafBase
+from gustaf.helpers._base import HelperBase
 
 
 class TrackedArray(np.ndarray):
-    """Taken from nice implementations of `trimesh` (see LICENSE.txt).
-    `https://github.com/mikedh/trimesh/blob/main/trimesh/caching.py`. Minor
-    adaption, since we don't have hashing functionalities.
-
-    All the inplace functions will set modified flag and if some operations
-    has potential to cause un-trackable behavior, writeable flags will be set
-    to False.
-
-    Note, if you really really want, it is possible to change the tracked
-    array without setting modified flag.
+    """numpy array object that keeps mirroring inplace changes to the source.
+    Meant to help control_points.
     """
 
-    __slots__ = ("_modified", "_source")
+    __slots__ = (
+        "_super_arr",
+        "_modified",
+    )
 
     def __array_finalize__(self, obj):
         """Sets default flags for any arrays that maybe generated based on
-        tracked array."""
+        physical space array. For more information,
+        see https://numpy.org/doc/stable/user/basics.subclassing.html"""
+        self._super_arr = None
         self._modified = True
-        self._source = int(0)
 
+        # for arrays created based on this subclass
         if isinstance(obj, type(self)):
-            if isinstance(obj._source, int):
-                self._source = obj
-            else:
-                self._source = obj._source
+            # this is copy. nothing to worry here
+            if self.base is None:
+                return None
+
+            # first child array
+            if self.base is obj:
+                # make sure this is not a recursively born child
+                # for example, `arr[[1,2]][:,2]`
+                # we should have set _super_arr to True
+                # if we made this array using `make_tracked_array`
+                if obj._super_arr is True:
+                    self._super_arr = obj
+                return None
+
+            # multi generation child array
+            if obj._super_arr is not None and self.base is obj.base:
+                self._super_arr = obj._super_arr
+                return None
+
+            return None
 
     @property
-    def mutable(self):
-        return self.flags["WRITEABLE"]
+    def modified(self):
+        """
+        Modified flag getter
+        """
+        # have super arr and self is not super_arr,
+        if self._super_arr is not None and self._super_arr is not True:
+            return self._super_arr._modified
 
-    @mutable.setter
-    def mutable(self, value):
-        self.flags.writeable = value
+        return self._modified
 
-    def _set_modified(self):
-        """set modified flags to itself and to the source."""
-        self._modified = True
-        if isinstance(self._source, type(self)):
-            self._source._modified = True
+    @modified.setter
+    def modified(self, m):
+        if self._super_arr is not None and self._super_arr is not True:
+            self._super_arr._modified = m
+        else:
+            self._modified = m
 
     def copy(self, *args, **kwargs):
-        """copy gives np.ndarray.
-
-        no more tracking.
-        """
-        return np.array(self, copy=True)
+        """copy creates regular numpy array"""
+        return np.array(self, *args, copy=True, **kwargs)
 
     def view(self, *args, **kwargs):
         """Set writeable flags to False for the view."""
@@ -67,89 +80,88 @@ class TrackedArray(np.ndarray):
         return v
 
     def __iadd__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__iadd__(*args, **kwargs)
+        sr = super(self.__class__, self).__iadd__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __isub__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__isub__(*args, **kwargs)
+        sr = super(self.__class__, self).__isub__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __imul__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__imul__(*args, **kwargs)
+        sr = super(self.__class__, self).__imul__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __idiv__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__idiv__(*args, **kwargs)
+        sr = super(self.__class__, self).__idiv__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __itruediv__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__itruediv__(*args, **kwargs)
+        sr = super(self.__class__, self).__itruediv__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __imatmul__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__imatmul__(*args, **kwargs)
+        sr = super(self.__class__, self).__imatmul__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __ipow__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__ipow__(*args, **kwargs)
+        sr = super(self.__class__, self).__ipow__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __imod__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__imod__(*args, **kwargs)
+        sr = super(self.__class__, self).__imod__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __ifloordiv__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__ifloordiv__(*args, **kwargs)
+        sr = super(self.__class__, self).__ifloordiv__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __ilshift__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__ilshift__(*args, **kwargs)
+        sr = super(self.__class__, self).__ilshift__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __irshift__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__irshift__(*args, **kwargs)
+        sr = super(self.__class__, self).__irshift__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __iand__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__iand__(*args, **kwargs)
+        sr = super(self.__class__, self).__iand__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __ixor__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__ixor__(*args, **kwargs)
+        sr = super(self.__class__, self).__ixor__(*args, **kwargs)
+        self.modified = True
+        return sr
 
     def __ior__(self, *args, **kwargs):
-        self._set_modified()
-        return super(self.__class__, self).__ior__(*args, **kwargs)
+        sr = super(self.__class__, self).__ior__(*args, **kwargs)
+        self.modified = True
+        return sr
 
-    def __setitem__(self, *args, **kwargs):
-        self._set_modified()
-        super(self.__class__, self).__setitem__(*args, **kwargs)
-
-    def __setslice__(self, *args, **kwargs):
-        self._set_modified()
-        super(self.__class__, self).__setslice__(*args, **kwargs)
-
-    def __getslice__(self, *args, **kwargs):
-        self._set_modified()
-        """
-        return slices I am pretty sure np.ndarray does not have __*slice__
-        """
-        slices = super(self.__class__, self).__getitem__(*args, **kwargs)
-        if isinstance(slices, np.ndarray):
-            slices.flags.writeable = False
-        return slices
+    def __setitem__(self, key, value):
+        # set first. invalid setting will cause error
+        sr = super(self.__class__, self).__setitem__(key, value)
+        self.modified = True
+        return sr
 
 
 def make_tracked_array(array, dtype=None, copy=True):
-    """Taken from nice implementations of `trimesh` (see LICENSE.txt).
+    """Motivated by nice implementations of `trimesh` (see LICENSE.txt).
     `https://github.com/mikedh/trimesh/blob/main/trimesh/caching.py`.
 
-    ``Properly subclass a numpy ndarray to track changes.
-    Avoids some pitfalls of subclassing by forcing contiguous
-    arrays and does a view into a TrackedArray.``
-
     Factory-like wrapper function for TrackedArray.
+    If you want to use TrackedArray, it is recommended to use this function.
 
     Parameters
     ------------
@@ -168,39 +180,33 @@ def make_tracked_array(array, dtype=None, copy=True):
     # if someone passed us None, just create an empty array
     if array is None:
         array = []
-    # make sure it is contiguous then view it as our subclass
-    tracked = np.ascontiguousarray(array, dtype=dtype)
-    if copy:
-        tracked = tracked.copy().view(TrackedArray)
-    else:
-        tracked = tracked.view(TrackedArray)
 
-    # should always be contiguous here
-    assert tracked.flags["C_CONTIGUOUS"]
+    if copy:
+        array = np.array(array, dtype=dtype)
+    else:
+        array = np.asanyarray(array, dtype=dtype)
+
+    tracked = array.view(TrackedArray)
+
+    # this marks original array
+    tracked._super_arr = True
 
     return tracked
 
 
-class DataHolder(GustafBase):
-    __slots__ = (
-        "_helpee",
-        "_saved",
-    )
+class DataHolder(HelperBase):
+    __slots__ = ("_saved",)
 
     def __init__(self, helpee):
         """Base class for any data holder. Behaves similar to dict.
 
-        Attributes
-        -----------
-        None
-
         Parameters
         -----------
         helpee: object
           GustafBase objects would probably make the most sense here.
         """
         self._helpee = helpee
-        self._saved = dict()
+        self._saved = {}
 
     def __setitem__(self, key, value):
         """Raise Error to disable direct value setting.
@@ -209,10 +215,6 @@ class DataHolder(GustafBase):
         -----------
         key: str
         value: object
-
-        Returns
-        --------
-        None
         """
         raise NotImplementedError(
             "Sorry, you can't set items directly for "
@@ -230,12 +232,39 @@ class DataHolder(GustafBase):
         --------
         value: object
         """
-        if key in self._saved.keys():
+        if key in self._saved:
             return self._saved[key]
 
         else:
             raise KeyError(f"`{key}` is not stored for {type(self._helpee)}")
 
+    def __contains__(self, key):
+        """Returns if saved data contains the given key.
+
+        Parameters
+        ----------
+        key: str
+
+        Returns
+        -------
+        value
+        """
+        return key in self._saved
+
+    def __len__(self):
+        """
+        Returns number of items.
+
+        Parameters
+        ----------
+        None
+
+        Returns
+        -------
+        len: int
+        """
+        return len(self._saved)
+
     def pop(self, key, default=None):
         """
         Applied pop() to saved data
@@ -254,16 +283,8 @@ class DataHolder(GustafBase):
     def clear(self):
         """
         Clears saved data by reassigning new dict
-
-        Parameters
-        ----------
-        None
-
-        Returns
-        -------
-        None
         """
-        self._saved = dict()
+        self._saved = {}
 
     def get(self, key, default_values=None):
         """Returns stored item if the key exists. Else, given default value. If
@@ -279,7 +300,7 @@ class DataHolder(GustafBase):
         --------
         value: object
         """
-        if key in self._saved.keys():
+        if key in self._saved:
             return self._saved[key]
         else:
             return default_values
@@ -287,10 +308,6 @@ class DataHolder(GustafBase):
     def keys(self):
         """Returns keys of data holding dict.
 
-        Parameters
-        -----------
-        None
-
         Returns
         --------
         keys: dict_keys
@@ -300,10 +317,6 @@ class DataHolder(GustafBase):
     def values(self):
         """Returns values of data holding dict.
 
-        Parameters
-        -----------
-        None
-
         Returns
         --------
         values: dict_values
@@ -313,16 +326,26 @@ class DataHolder(GustafBase):
     def items(self):
         """Returns items of data holding dict.
 
-        Parameters
-        -----------
-        None
-
         Returns
         --------
         values: dict_values
         """
         return self._saved.items()
 
+    def update(self, **kwargs):
+        """
+        Updates given kwargs using __setitem__.
+
+        Parameters
+        ----------
+        **kwargs: kwargs
+
+        Returns
+        -------
+        None
+        """
+        self._saved.update(**kwargs)
+
 
 class ComputedData(DataHolder):
     _depends = None
@@ -330,13 +353,11 @@ class ComputedData(DataHolder):
 
     __slots__ = ()
 
-    def __init__(self, helpee, **kwargs):
-        """Stores last computed values. Keys are expected to be the same as
-        helpee's function that computes the value.
+    def __init__(self, helpee, **_kwargs):
+        """Stores last computed values.
 
-        Attributes
-        -----------
-        None
+        Keys are expected to be the same as helpee's function that computes the
+         value.
 
         Parameters
         -----------
@@ -364,26 +385,26 @@ class ComputedData(DataHolder):
         """
 
         def inner(func):
-            # followings are done once while modules are loaded
+            # following are done once while modules are loaded
             # just subclass this class to make a special helper
             # for each helpee class.
             assert isinstance(var_names, list), "var_names should be a list"
             # initialize property
             # _depends is dict(str: list)
             if cls._depends is None:
-                cls._depends = dict()
+                cls._depends = {}
             if cls._depends.get(func.__name__, None) is None:
-                cls._depends[func.__name__] = list()
+                cls._depends[func.__name__] = []
             # add dependency info
             cls._depends[func.__name__].extend(var_names)
 
             # _inv_depends is dict(str: list)
             if cls._inv_depends is None:
-                cls._inv_depends = dict()
+                cls._inv_depends = {}
             # add inverse dependency
             for vn in var_names:
                 if cls._inv_depends.get(vn, None) is None:
-                    cls._inv_depends[vn] = list()
+                    cls._inv_depends[vn] = []
 
                 cls._inv_depends[vn].append(func.__name__)
 
@@ -392,17 +413,18 @@ class ComputedData(DataHolder):
                 """Check if the key should be computed,"""
                 # extract some related info
                 self = args[0]  # the helpee itself
-                recompute = kwargs.get("recompute", False)
-                return_saved = kwargs.get("return_saved", False)
 
-                # if return_saved, try to escape as soon as possible
-                if return_saved:
-                    saved = self._computed._saved.get(func.__name__, None)
-                    if saved is not None and not recompute:
-                        return saved
+                # explicitly settable kwargs.
+                # unless recompute flag is set False,
+                # it will always recompute and save them
+                # if you call the same function without kwargs
+                # the last one with kwargs will be returned
+                recompute = False
+                if kwargs:
+                    recompute = kwargs.get("recompute", True)
 
                 # computed arrays are called _computed.
-                # loop over dependees and check if they are modified
+                # loop over dependencies and check if they are modified
                 for dependee_str in cls._depends[func.__name__]:
                     dependee = getattr(self, dependee_str)
                     # is modified?
@@ -451,17 +473,12 @@ class VertexData(DataHolder):
     __slots__ = ()
 
     def __init__(self, helpee):
-        """
-        Checks if helpee has vertices as attr beforehand.
+        """Checks if helpee has vertices as attr beforehand.
 
         Parameters
         ----------
         helpee: Vertices
           Vertices and its derived classes.
-
-        Returns
-        -------
-        None
         """
         if not hasattr(helpee, "vertices"):
             raise AttributeError("Helpee does not have `vertices`.")
@@ -469,8 +486,8 @@ class VertexData(DataHolder):
         super().__init__(helpee)
 
     def _validate_len(self, value=None, raise_=True):
-        """
-        Checks if given value is a valid vertexdata based of its length.
+        """Checks if given value is a valid vertex_data based of its length.
+
         If raise_, throws error, else, deletes all incompatible values.
         Only checks len(). If array has (1, len) shape, this will still return
         False.
@@ -496,12 +513,13 @@ class VertexData(DataHolder):
             if raise_ and not valid:
                 raise ValueError(
                     f"Expected ({helpee_len}) length data, "
-                    f"Given (len(value))"
+                    f"Given ({len(value)})"
                 )
 
             return valid
 
         # here, check all saved values.
+        to_pop = []
         for key, value in self._saved.items():
             if len(value) != helpee_len:
                 valid = False
@@ -515,17 +533,21 @@ class VertexData(DataHolder):
                 else:
                     self._logd(
                         f"`{key}`-data len ({len(value)}) doesn't match "
-                        f"expect len ({helpee_len}). Deleting `{key}`."
+                        f"expected len ({helpee_len}). Deleting `{key}`."
                     )
                 # pop invalid data
-                self._saved.pop(key)
-                self._saved.pop(key + "__norm", None)
+                to_pop.append(key)
+                to_pop.append(key + "__norm")
+
+        # pop if needed
+        for tp in to_pop:
+            self._saved.pop(tp, None)
 
         return valid
 
     def __setitem__(self, key, value):
         """
-        Performs len() based check before stroing vertexdata.
+        Performs len() based check before storing vertex_data.
 
         Parameters
         ----------
@@ -543,6 +565,14 @@ class VertexData(DataHolder):
             len(self._helpee.vertices), -1
         )
 
+        # if "data" or "arrow_data" is empty in show_options, we want to
+        # set this data to show. We will always set this as "data".
+        show_options = getattr(self._helpee, "show_options", None)
+        if show_options is not None:
+            if "data" in show_options or "arrow_data" in show_options:
+                return None
+            show_options["data"] = key
+
     def __getitem__(self, key):
         """
         Validates data length before returning item.
@@ -585,18 +615,18 @@ class VertexData(DataHolder):
         # interpret scalar as norm
         # save the norm once it is called.
         if "__norm" not in key:
-            normkey = key + "__norm"
+            norm_key = key + "__norm"
         else:
-            normkey = key
+            norm_key = key
             key = key.replace("__norm", "")
 
-        if normkey in self.keys():
-            saved = self[normkey]  # performs len check
+        if norm_key in self.keys():
+            saved = self[norm_key]  # performs len check
             # return if original is not modified
             if not self[key]._modified:  # check if original data is modified
                 return saved
             else:
-                self._saved.pop(normkey)
+                self._saved.pop(norm_key)
 
         # we are here because we have to compute norm. let's save norm
         value = self[key]
@@ -606,7 +636,7 @@ class VertexData(DataHolder):
             value_norm = np.linalg.norm(value, axis=1).reshape(-1, 1)
 
         # save norm
-        self[normkey] = value_norm
+        self[norm_key] = value_norm
         # considered not modified
         self[key]._modified = False
 
@@ -640,237 +670,6 @@ class VertexData(DataHolder):
         return value
 
 
-class SplineDataAdaptor(GustafBase):
-    """
-    Prepares data to be presentable on spline. To support both
-    scalar-data and vector-data, which are representable with colors and
-    arrows respectively, this class will prepare data accordingly.
-    """
-
-    __slots__ = (
-        "data",
-        "function",
-        "locations",
-        "is_spline",
-        "has_function",
-        "has_locations",
-        "has_evaluate",
-        "arrowdata_only",
-        "_user_created",
-    )
-
-    def __init__(self, data, locations=None, function=None):
-        """ """
-        # default
-        self._user_created = True
-        self.data = data
-        self.is_spline = False
-        self.has_function = False
-        self.has_locations = False
-        self.has_evaluate = False
-        self.arrowdata_only = False
-
-        # is spline we know?
-        if "CoreSpline" in str(type(data).__mro__):
-            self.is_spline = True
-
-        # data has evaluate?
-        if hasattr(data, "evaluate"):
-            self.has_evaluate = callable(data.evaluate)
-
-        # has function?
-        if function is not None:
-            self.has_function = True
-            if not callable(function):
-                raise ValueError("Given function isn't callable")
-            self.function = function
-
-        # locations? - keep this compatible with functions. maybe
-        # we want to have some state dependent value at certain locations
-        if locations is not None:
-            # set what holds true
-            self.has_locations = True
-            self.arrowdata_only = True
-            self.locations = np.asanyarray(locations)
-
-            # if this is not a spline we know, it doesn't have a function,
-            # it should:
-            # -> `data.evaluate` is callable, or
-            # -> len(data) == len(locations)
-            if not self.is_spline and not self.has_function:
-                len_matches = False
-                if hasattr(data, "__len__"):
-                    len_matches = len(locations) == len(data)
-                if not (self.has_evaluate or len_matches):
-                    raise ValueError(
-                        "Data cannot be represented at specified locations."
-                        "Requires one of the following requirements: "
-                        "1) is a spline derived from splinepy's spline; "
-                        "2) data has `data.evalauate()`; "
-                        "3) length of the data and location should match."
-                    )
-            # location is sepcified, meaning we don't need sample()
-            return None
-
-        # can call sample or has a function?
-        if not self.has_function and not self.is_spline:
-            raise ValueError(
-                "None spline data should at least have an accompanying "
-                "function."
-            )
-
-    def as_vertexdata(self, resolutions=None, on=None):
-        """
-        Parameters
-        ----------
-        resolutions: list or tuple
-        at: (n, d) array-like
-
-        Returns
-        -------
-        vertexdata: (m, r) array-like
-        """
-        if resolutions is not None and on is not None:
-            raise ValueError(
-                "Please only specify either `resolutions` or `on`"
-            )
-
-        if self.has_locations and (resolutions is not None or on is not None):
-            raise ValueError(
-                "Location dependent data can't be evaluated with `resolutions`"
-                " or `at`."
-            )
-
-        # if resolutions is specified, this is not a location query
-        if resolutions is not None:
-            if self.has_function:
-                return self.function(self.data, resolutions=resolutions)
-            elif self.is_spline and self.data.para_dim > 2:
-                # TODO: replace this with generalized query helpers.
-                return self.data.extract.faces(resolutions).vertices
-            else:
-                return self.data.sample(resolutions)
-
-        # runtime location query
-        if on is not None:
-            if self.has_function:
-                return self.function(self.data, on=on)
-            elif self.has_evaluate:
-                return self.data.evaluate(on)
-            else:
-                raise ValueError(
-                    "Given data can't support data extraction on specified "
-                    f"locations ({on})."
-                )
-
-        # location specified - either evaluate function at the
-        if self.has_locations:
-            if self.has_function:
-                # function may want locations
-                try:
-                    return self.function(self.data, self.locations)
-                except TypeError:  # maybe too many args
-                    return self.function(self.data)
-            elif self.has_evaluate:
-                return self.data.evaluate(self.locations)
-            else:
-                return self.data
-
-        # should be returned by now
-        raise RuntimeError("Something went wrong while preparing spline data.")
-
-
-class SplineData(DataHolder):
-    """
-    Data manager for splines.
-    """
-
-    def __init__(self, helpee):
-        """ """
-        if "GustafSpline" not in str(type(helpee).__mro__):
-            raise AttributeError("Helpee does not have `vertices`")
-
-        super().__init__(helpee)
-
-    def __setitem__(self, key, value):
-        """
-        Selectively accept spline data.
-
-        Parameters
-        ----------
-        key: str
-        value: object
-
-        Returns
-        -------
-        None
-        """
-        if isinstance(value, SplineDataAdaptor):
-            self._saved[key] = value
-        else:
-            adapted = SplineDataAdaptor(value)  # will test usability
-            adapted._user_created = False  # mark for __getitem__
-            self._saved[key] = adapted
-
-    def __getitem__(self, key):
-        """
-        Returns value from __setitem__
-
-        Parameters
-        ----------
-        key: str
-
-        Returns
-        -------
-        value: object
-        """
-        saved = super().__getitem__(key)
-        if saved._user_created:
-            return saved
-        else:
-            return saved.data
-
-    def as_scalar(self, key, resolutions, default=None):
-        """
-        Return scalar value at given resolutions
-
-        Parameters
-        ----------
-        key: str
-        resolutions: list or tuple
-        default: object
-          Default is None and will return is key doesn't exist
-
-        Returns
-        -------
-        value: np.ndarray
-        """
-        if key not in self._saved:
-            return default
-
-        saved = super().__getitem__(key)
-        # will raise
-        return saved.as_vertexdata(resolutions=resolutions)
-
-    def as_arrow(self, key, resolutions=None, on=None, default=None):
-        """
-        Returns as-arrow-representable data on certain places, with given
-        resolution, or on predefined places.
-
-        Parameters
-        ----------
-        key: str
-        resolutions: list or tuple
-        on: array-like
-        """
-        if key not in self._saved:
-            return default
-
-        saved = super().__getitem__(key)
-        # will raise
-        return saved.as_vertexdata(resolutions=resolutions, on=on)
-
-
 Unique2DFloats = namedtuple(
     "Unique2DFloats", ["values", "ids", "inverse", "intersection"]
 )
