@@ -8,14 +8,21 @@ import struct
 
 import numpy as np
 
+from gustaf import settings
 from gustaf.faces import Faces
 from gustaf.io.ioutils import abs_fname, check_and_makedirs
 from gustaf.utils import log
+from gustaf.utils.arr import close_rows
 from gustaf.volumes import Volumes
 
 
 def load(
-    simplex=True, volume=False, fname=None, mxyz=None, mien=None, mrng=None
+    simplex=True,
+    volume=False,
+    fname=None,
+    mxyz=None,
+    mien=None,
+    mrng=None,
 ):
     """mixd load. To avoid reading minf, all the crucial info can be given as
     params. Default input will try to import `mxyz`, `mien`, `mrng` from
@@ -71,20 +78,20 @@ def load(
     # connec
     connec = None
     try:
-        connec = (np.fromfile(mien, dtype=">i") - int(1)).astype(np.int32)
+        connec = (np.fromfile(mien, dtype=">i") - 1).astype(np.int32)
     except BaseException:
         log.debug(f"mien file, `{mien}`, does not exist. Skipping.")
 
     # boundary conditions
-    bcs = dict()
+    bcs = {}
     try:
         bcs_in = np.fromfile(mrng, dtype=">i").astype(np.int32)  # flattened
         uniq_bcs_in = np.unique(bcs_in)
         uniq_bcs_in = uniq_bcs_in[uniq_bcs_in > 0]  # keep only natural nums
-        subelemids = np.arange(bcs_in.size)
+        sub_elem_ids = np.arange(bcs_in.size)
 
         for ubci in uniq_bcs_in:
-            bcs.update({str(ubci): subelemids[bcs_in == ubci]})
+            bcs.update({str(ubci): sub_elem_ids[bcs_in == ubci]})
 
     except BaseException:
         log.debug(f"mrng file, `{mrng}`, does not exist. Skipping.")
@@ -94,8 +101,8 @@ def load(
 
     # reshape connec
     if connec is not None:
-        ncol = int(3) if simplex and not volume else int(4)
-        ncol = int(8) if ncol == int(4) and volume and not simplex else ncol
+        ncol = 3 if simplex and not volume else 4
+        ncol = 8 if ncol == 4 and volume and not simplex else ncol
 
         connec = connec.reshape(-1, ncol)
 
@@ -109,9 +116,10 @@ def load(
 
 
 def export(
-    mesh,
     fname,
+    mesh,
     space_time=False,
+    dual=False,
 ):
     """Export in mixd format. Supports triangle, quadrilateral, tetrahedron,
     and hexahedron semi-discrete and (flat) space-time mesh output.
@@ -120,8 +128,10 @@ def export(
     -----------
     mesh: Faces or Volumes
     fname: str
-    space_time : bool
-      Export Mesh as Space-Time Slab for discontinous space-time
+    space_time: bool
+      Export Mesh as Space-Time Slab for discontinuous space-time
+    dual: bool
+      Includes dual-subelement information.
 
     Returns
     --------
@@ -156,6 +166,7 @@ def export(
             connec_file = os.path.join(fdir, "mien")
             bc_file = os.path.join(fdir, "mrng")
             info_file = os.path.join(fdir, "minf")
+            dual_file = os.path.join(fdir, "dual")
 
         else:
             fbase += "."
@@ -163,6 +174,7 @@ def export(
             connec_file = fbase + "mien"
             bc_file = fbase + "mrng"
             info_file = fbase + "minf"
+            dual_file = fbase + "dual"
 
     else:
         raise NotImplementedError("`mixd` format only supports xns.")
@@ -181,13 +193,62 @@ def export(
         for c in mesh.elements.ravel() + 1:
             cf.write(struct.pack(big_endian_int, c))
 
-    # write bc
-    with open(bc_file, "wb") as bf:
-        boundaries = make_mrng(mesh)
+    # get boundaries of each element - subelement interface array
+    sub_interface = make_mrng(mesh)
 
-        for b in boundaries:
+    # write bc first - after writing it, we can modify inplace for dual.
+    with open(bc_file, "wb") as bf:
+        for b in sub_interface:
             bf.write(struct.pack(big_endian_int, b))
 
+    # if dual is True, we fill dual infos.
+    if dual:
+        sub_elements = mesh.to_subelements(False)
+        # this should be always dividable without remnants
+        n_subelem_per_elem, rem = divmod(
+            len(sub_elements.elements), len(mesh.elements)
+        )
+        if rem != 0:
+            raise ValueError(
+                "something went wrong with subelement creation."
+                "Please report this issue, thank you!"
+            )
+
+        # get intersection - can use this info to determine duals
+        _, _, _, intersections = close_rows(
+            sub_elements.centers(), settings.TOLERANCE, True
+        )
+
+        # loop intersections and look for 2 intersections
+        for i, intersection in enumerate(intersections):
+            n_inter = len(intersection)
+
+            # we modify interface only if there're 2 intersections.
+            if n_inter == 2:
+                # intersection is always sorted.
+                # we don't want dual to point to itself
+                dual_id = 0 if i != intersection[0] else 1
+
+                # get element number and apply fortran's offset, 1
+                sub_interface[i] = -int(
+                    intersection[dual_id] // n_subelem_per_elem + 1
+                )
+                continue
+
+            # intersection should be at most 2. Otherwise, it either means
+            # that you have a bad mesh or to big tolerance
+            if n_inter > 2:
+                raise ValueError(
+                    f"{i}-th subelement overlaps more than once. "
+                    "Please check your elements or decrease "
+                    "gustaf.settings.TOLERANCE."
+                )
+
+        # write dual
+        with open(dual_file, "wb") as df:
+            for d in sub_interface:
+                df.write(struct.pack(big_endian_int, d))
+
     # write info
     with open(info_file, "w") as infof:  # if and inf... just can't
         infof.write(f"# dim: {dim}\n")
@@ -229,7 +290,7 @@ def make_mrng(mesh):
       The mrng-array.
     """
 
-    # determine number of subelements
+    # determine number of sub elements
     whatami = mesh.whatami
     nbelem = 3
 
@@ -238,11 +299,10 @@ def make_mrng(mesh):
     elif whatami.startswith("hexa"):
         nbelem += 3
 
-    # init boundaries with -1, as it is the value for non-boundary.
-    # alternatively, they could be (-1 * neighbor_elem_id).
-    # But they aren't.
+    # init boundaries with 0, as boundaries are marked with positive numbers
+    # staring from 1 and dual infos with negative values, starting with -1
     boundaries = np.empty(mesh.elements.shape[0] * nbelem, dtype=int)
-    boundaries[:] = -1
+    boundaries[:] = 0
 
     for i, belem_ids in enumerate(mesh.BC.values()):
         boundaries[belem_ids] = i + 1  # bid starts at 1
