@@ -1,4 +1,5 @@
 from gustaf.utils import arr, connec, log, tictoc
+from gustaf.utils.tictoc import Tic
 
 # Alias
 connectivity = connec
@@ -9,4 +10,5 @@ __all__ = [
     "connectivity",
     "log",
     "tictoc",
+    "Tic",
 ]
